@@ -1,42 +1,46 @@
-/* Copyright 2016 The Tor Project
+/* Copyright 2016--2017 The Tor Project
  * See LICENSE for licensing information */
+
 package org.torproject.descriptor;
 
-import static org.torproject.descriptor.DescriptorSourceFactory.*;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+import static org.torproject.descriptor.DescriptorSourceFactory.COLLECTOR_DEFAULT;
+import static org.torproject.descriptor.DescriptorSourceFactory.COLLECTOR_PROPERTY;
+import static org.torproject.descriptor.DescriptorSourceFactory.DOWNLOADER_DEFAULT;
+import static org.torproject.descriptor.DescriptorSourceFactory.DOWNLOADER_PROPERTY;
+import static org.torproject.descriptor.DescriptorSourceFactory.PARSER_DEFAULT;
+import static org.torproject.descriptor.DescriptorSourceFactory.PARSER_PROPERTY;
+import static org.torproject.descriptor.DescriptorSourceFactory.READER_DEFAULT;
+import static org.torproject.descriptor.DescriptorSourceFactory.READER_PROPERTY;
 
-import org.torproject.descriptor.DescriptorCollector;
-import org.torproject.descriptor.DescriptorDownloader;
-import org.torproject.descriptor.DescriptorParser;
-import org.torproject.descriptor.DescriptorReader;
-import org.torproject.descriptor.ImplementationNotAccessibleException;
 import org.torproject.descriptor.impl.DescriptorDownloaderImpl;
 import org.torproject.descriptor.impl.DescriptorParserImpl;
 import org.torproject.descriptor.impl.DescriptorReaderImpl;
 import org.torproject.descriptor.index.DescriptorIndexCollector;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
-
 import org.junit.Test;
 
-import java.lang.reflect.Method;
 import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 
 public class DescriptorSourceFactoryTest {
 
-  private final static String[] properties = new String[]{
-      COLLECTOR_PROPERTY, DOWNLOADER_PROPERTY, PARSER_PROPERTY, READER_PROPERTY};
-  private final static String[] defaults = new String[]{
-      COLLECTOR_DEFAULT, DOWNLOADER_DEFAULT, PARSER_DEFAULT, READER_DEFAULT};
+  private static final String[] properties = new String[] { COLLECTOR_PROPERTY,
+      DOWNLOADER_PROPERTY, PARSER_PROPERTY, READER_PROPERTY };
+
+  private static final String[] defaults = new String[] { COLLECTOR_DEFAULT,
+      DOWNLOADER_DEFAULT, PARSER_DEFAULT, READER_DEFAULT };
 
   @Test()
   public void testDefaults() {
     setProperties(defaults);
-    DescriptorCollector dc = DescriptorSourceFactory.createDescriptorCollector();
+    DescriptorCollector dc =
+        DescriptorSourceFactory.createDescriptorCollector();
     assertTrue(dc instanceof DescriptorIndexCollector);
-    DescriptorDownloader dd = DescriptorSourceFactory.createDescriptorDownloader();
+    DescriptorDownloader dd =
+        DescriptorSourceFactory.createDescriptorDownloader();
     assertTrue(dd instanceof DescriptorDownloaderImpl);
     DescriptorParser dp = DescriptorSourceFactory.createDescriptorParser();
     assertTrue(dp instanceof DescriptorParserImpl);
@@ -66,7 +70,7 @@ public class DescriptorSourceFactoryTest {
       retrieve.setAccessible(true);
       retrieve.invoke(null, "unknown.property");
     } catch (InvocationTargetException ite) {
-      if(ite.getCause() instanceof ImplementationNotAccessibleException) {
+      if (ite.getCause() instanceof ImplementationNotAccessibleException) {
         return;
       } else {
         fail("Cause was " + ite.getCause()
@@ -79,16 +83,17 @@ public class DescriptorSourceFactoryTest {
 
   @Test()
   public void testProperties() {
-    setProperties(new String[]{
+    setProperties(new String[] {
         "org.torproject.descriptor.DummyCollectorImplementation",
         "org.torproject.descriptor.DummyDownloaderImplementation",
         "org.torproject.descriptor.DummyParserImplementation",
-        "org.torproject.descriptor.DummyReaderImplementation",
-      });
-    DescriptorCollector dc = DescriptorSourceFactory.createDescriptorCollector();
+        "org.torproject.descriptor.DummyReaderImplementation" });
+    DescriptorCollector dc =
+        DescriptorSourceFactory.createDescriptorCollector();
     assertTrue(dc instanceof DummyCollectorImplementation);
     assertEquals(1, DummyCollectorImplementation.count);
-    DescriptorDownloader dd = DescriptorSourceFactory.createDescriptorDownloader();
+    DescriptorDownloader dd =
+        DescriptorSourceFactory.createDescriptorDownloader();
     assertTrue(dd instanceof DummyDownloaderImplementation);
     assertEquals(1, DummyDownloaderImplementation.count);
     DescriptorParser dp = DescriptorSourceFactory.createDescriptorParser();
@@ -98,33 +103,4 @@ public class DescriptorSourceFactoryTest {
     assertTrue(dr instanceof DummyReaderImplementation);
     assertEquals(1, DummyReaderImplementation.count);
   }
-
-}
-
-class DummyCollectorImplementation extends DescriptorIndexCollector {
-  static int count;
-  public DummyCollectorImplementation() {
-    count++;
-  }
-}
-
-class DummyDownloaderImplementation extends DescriptorDownloaderImpl {
-  static int count;
-  public DummyDownloaderImplementation() {
-    count++;
-  }
-}
-
-class DummyParserImplementation extends DescriptorParserImpl {
-  static int count;
-  public DummyParserImplementation() {
-    count++;
-  }
-}
-
-class DummyReaderImplementation extends DescriptorReaderImpl {
-  static int count;
-  public DummyReaderImplementation() {
-    count++;
-  }
 }
